(******************************************************************************
 *                                                                            *
 *      ,  2002                                 *
 *                                                                            *
 *  :       basedd8.pas                                                   *
 *  :       DirectDraw API               *
 *                                                                            *
 ******************************************************************************)
unit basedd8;

interface

uses
  windows,
  directdraw;




//------------------------------------------------------------------------------
//    
//------------------------------------------------------------------------------
const
  SCREEN_WIDTH  = 640; //  
  SCREEN_HEIGHT = 480; //  




//------------------------------------------------------------------------------
//  
//------------------------------------------------------------------------------
function InitDirectDraw( hWnd: HWND; dwWidth, dwHeight, dwBPP: DWORD ): Boolean;
procedure ReleaseDirectDraw();
function CopyImage( var lpSurf: IDirectDrawSurface7; hImage: HBITMAP ): Boolean;
function LoadFiles( hWnd: HWND ): Boolean;
procedure OnDraw( nX, nY: Integer );

implementation




//------------------------------------------------------------------------------
//    
//------------------------------------------------------------------------------
var
  lpDD7:            IDirectDraw7        = nil;  //   DirectDraw
  lpPrimarySurface: IDirectDrawSurface7 = nil;  //  
  lpBackBuffer:     IDirectDrawSurface7 = nil;  //  

  lpWallpaper:      IDirectDrawSurface7 = nil;  //  
  lpSprite:         IDirectDrawSurface7 = nil;  // 




//------------------------------------------------------------------------------
// :      InitDirectDraw()
// :    DirectDraw  
//------------------------------------------------------------------------------
function InitDirectDraw( hWnd: HWND; dwWidth, dwHeight, dwBPP: DWORD ): Boolean;
var
  ddsd2: TDDSURFACEDESC2;
  ddsc2: TDDSCAPS2;
begin
  Result := FALSE;

  //    DirectDraw
  if FAILED( DirectDrawCreateEx( nil, lpDD7, IID_IDIRECTDRAW7, nil ) ) then
     Exit;
  lpDD7._AddRef();

  //   
  if FAILED( lpDD7.SetCooperativeLevel( hWnd, DDSCL_ALLOWREBOOT or
                                              DDSCL_EXCLUSIVE or
                                              DDSCL_FULLSCREEN ) ) then
     Exit;

  //      
  if FAILED( lpDD7.SetDisplayMode( dwWidth, dwHeight, dwBpp, 0, 0 ) ) then
     Exit;

  //   TDDSURFACEDESC2
  ZeroMemory( @ddsd2, SizeOf( TDDSURFACEDESC2 ) );
  ddsd2.dwSize := SizeOf( TDDSURFACEDESC2 );

  ddsd2.dwFlags := DDSD_CAPS or DDSD_BACKBUFFERCOUNT;
  ddsd2.ddsCaps.dwCaps := DDSCAPS_PRIMARYSURFACE or
                          DDSCAPS_FLIP or
                          DDSCAPS_COMPLEX;
  ddsd2.dwBackBufferCount := 1;

  //    (    )
  if FAILED( lpDD7.CreateSurface( ddsd2, lpPrimarySurface, nil ) ) then
     Exit;
  lpPrimarySurface._AddRef();

  //   TDDSCAPS2
  ZeroMemory( @ddsc2, SizeOf( TDDSCAPS2 ) );
  ddsc2.dwCaps := DDSCAPS_BACKBUFFER; //,    

  //      
  if FAILED( lpPrimarySurface.GetAttachedSurface( ddsc2, lpBackBuffer ) ) then
     Exit;
  lpBackBuffer._AddRef();

  Result := TRUE;
end;




//------------------------------------------------------------------------------
// :      ReleaseDirectDraw()
// :     DirectDraw
//------------------------------------------------------------------------------
procedure ReleaseDirectDraw();
begin
  //    
  if lpBackBuffer <> nil then
  begin
    lpBackBuffer._Release();
    lpBackBuffer := nil;
  end;

  //     
  if lpPrimarySurface <> nil then
  begin
    lpPrimarySurface._Release();
    lpPrimarySurface := nil;
  end;

  // ,     DirectDraw
  if lpDD7 <> nil then
  begin
    lpDD7.RestoreDisplayMode(); //   
    lpDD7._Release();
    lpDD7 := nil;
  end;
end;




//------------------------------------------------------------------------------
// :      CreateSurface()
// :   DirectDraw  
//------------------------------------------------------------------------------
function CreateSurface( var lpSurface: IDirectDrawSurface7; nWidth,
                        nHeight: Integer ): Boolean;
var
  ddsd2: TDDSURFACEDESC2;
begin
  Result := FALSE;

  //   ,  
  if lpSurface <> nil then
  begin
    lpSurface._Release;
    lpSurface := nil;
  end;

  //   TDDSURFACEDESC2
  ZeroMemory( @ddsd2, SizeOf( TDDSURFACEDESC2 ) );
  with ddsd2 do
  begin
    dwSize         := SizeOf( TDDSURFACEDESC2 );
    dwFlags        := DDSD_CAPS or DDSD_WIDTH or DDSD_HEIGHT;
    ddsCaps.dwCaps := DDSCAPS_OFFSCREENPLAIN;
    dwWidth        := nWidth;
    dwHeight       := nHeight;
  end;

  //  
  if FAILED( lpDD7.CreateSurface( ddsd2, lpSurface, nil ) ) then
     Exit;
  lpSurface._AddRef;

  Result := TRUE;
end;




//------------------------------------------------------------------------------
// :      CopyImage()
// :     DirectDraw
//------------------------------------------------------------------------------
function CopyImage( var lpSurf: IDirectDrawSurface7; hImage: HBITMAP ): Boolean;
var
  destDC,          // -
  srcDC:  HDC;     // -
  bm:     BITMAP;  // BITMAP  windows.pas
  hObj:   HGDIOBJ; //  GDI
begin
  Result := FALSE;

  //   ,    
  srcDC := CreateCompatibleDC( 0 );

  //     -  
  if srcDC = 0 then Exit;

  //    
  hObj := SelectObject( srcDC, hImage );
  if ( hObj = 0 ) or ( hObj = GDI_ERROR ) then
     Exit;

  //   ,    BITMAP
  if GetObject( hImage, SizeOf( bm ), @bm ) = 0 then
     Exit;

  //     
  if not CreateSurface( lpSurf, bm.bmWidth, bm.bmHeight ) then
     Exit;

  //  GDI-   
  if SUCCEEDED( lpSurf.GetDC( destDC ) ) then
  begin
    // BitBlt-   (    DirectDraw)
    BitBlt( destDC, 0, 0, bm.bmWidth, bm.bmHeight, srcDC, 0, 0, SRCCOPY );
    lpSurf.ReleaseDC( destDC );
  end
  else Exit;

  //      GDI
  DeleteObject( hObj );
  DeleteDC( srcDC );

  Result := TRUE;
end;




//------------------------------------------------------------------------------
// :      LoadFiles()
// :    bmp-   DirectDraw
//------------------------------------------------------------------------------
function LoadFiles( hWnd: HWND ): Boolean;
var
  hImage: HBITMAP;
begin
   Result := FALSE;

   //      
   hImage := LoadImage( 0, 'wallpaper.bmp', IMAGE_BITMAP, 0, 0,
                        LR_LOADFROMFILE or LR_CREATEDIBSECTION );
   if hImage = 0 then
   begin
     MessageBox( hWnd, '   wallpaper.bmp!', '!', MB_ICONHAND );
     Exit;
   end;

   //     DirectDraw
   if not CopyImage( lpWallpaper, hImage ) then
      Exit;

   //     (     DSK).
   hImage := LoadImage( 0, 'sprite.bmp', IMAGE_BITMAP, 0, 0,
                        LR_LOADFROMFILE or LR_CREATEDIBSECTION );
   if hImage = 0 then
   begin
     MessageBox( hWnd, '   sprite.bmp!', '!', MB_ICONHAND );
     Exit;
   end;

   if not CopyImage( lpSprite, hImage ) then
      Exit;

   Result := TRUE;
end;




//------------------------------------------------------------------------------
// :      OnDraw()
// :       
//------------------------------------------------------------------------------
procedure OnDraw( nX, nY: Integer );
var
  rRect: TRect;
begin
  //      
  SetRect( rRect, 0, 0, SCREEN_WIDTH, SCREEN_HEIGHT );
  //      
  lpBackBuffer.BltFast( 0, 0, lpWallpaper, @rRect, DDBLTFAST_WAIT );

  //   
  SetRect( rRect, 0, 0, 256, 256 );
  lpBackBuffer.BltFast( nX, nY, lpSprite, @rRect, DDBLTFAST_WAIT );

  //         
  lpPrimarySurface.Flip( nil, DDFLIP_WAIT );
end;

end.
