(******************************************************************************
 *                                                                            *
 *      ,  2002                                 *
 *                                                                            *
 *  :       main.pas                                                      *
 *  :   DirectSound       *
 *                                                                            *
 ******************************************************************************)
unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Buttons;

type
  TForm1 = class(TForm)
    btnOpen: TButton;
    odOpenFile: TOpenDialog;
    edFileName: TEdit;
    chbLoop: TCheckBox;
    bbPlay: TBitBtn;
    bbStop: TBitBtn;
    sbVolume: TScrollBar;
    sbPan: TScrollBar;
    Timer1: TTimer;
    procedure btnOpenClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure bbPlayClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure bbStopClick(Sender: TObject);
    procedure sbVolumeChange(Sender: TObject);
    procedure sbPanChange(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses
  WavRead,     //      wav-
  DirectSound;




//------------------------------------------------------------------------------
//    
//------------------------------------------------------------------------------
var
  lpDS8:         IDirectSound8;
  lpSoundBuffer: IDirectSoundBuffer;
  bbytes:        Integer;            //   

  wsr:           TWaveSoundRead;     //      wav-




//------------------------------------------------------------------------------
// :      InitDirectSound()
// :   DirectSound  
//------------------------------------------------------------------------------
function InitDirectSound( hWnd: HWND ): Boolean;
begin
  Result := FALSE;

  //    DirectSound
  if FAILED( DirectSoundCreate8( nil, lpDS8, nil ) ) then
     Exit;
  lpDS8._AddRef();

  //   .  -  
  if FAILED( lpDS8.SetCooperativeLevel( hWnd, DSSCL_PRIORITY ) ) then
     Exit;

  Result := TRUE;
end;




//------------------------------------------------------------------------------
// :      ReleaseDirectSound()
// :    DirectSound
//------------------------------------------------------------------------------
procedure ReleaseDirectSound();
begin
  //   
  if lpSoundBuffer <> nil then
  begin
    lpSoundBuffer._Release;
    lpSoundBuffer := nil;
  end;

  //     DirectSound
  if lpDS8 <> nil then
  begin
    lpDS8._Release();
    lpDS8 := nil;
  end;
end;




//------------------------------------------------------------------------------
// :      CreateStaticBuffer()
// :    DirectSound    
//------------------------------------------------------------------------------
function CreateStaticBuffer( strFileName: string ): Boolean;
var
  dsbd8: TDSBUFFERDESC;
begin
  Result := FALSE;

  //     
  if wsr <> nil then wsr.Destroy();

  //    ,  
  if lpSoundBuffer <> nil then
  begin
    lpSoundBuffer._Release;
    lpSoundBuffer := nil;
  end;

  //        
  wsr := TWaveSoundRead.Create();
  if not wsr.Open( strFileName ) then
     Exit;

  //   TDSBUFFERDESC_DX8
  ZeroMemory( @dsbd8, SizeOf( TDSBUFFERDESC ) );
  dsbd8.dwSize  := SizeOf( TDSBUFFERDESC );
  dsbd8.dwFlags := DSBCAPS_STATIC or     //     
                   DSBCAPS_CTRLPAN or    //     
                   DSBCAPS_CTRLVOLUME or //     
                   DSBCAPS_GLOBALFOCUS;  //   
  // :      DSBCAPS_LOCHARDWARE  
  //  ,      .
  //     CreateSoundBuffer()   

  //      TWaveSoundRead -    
  //     ,     -
  //      WAVEFORMATEX.   
  // DirectSound -    ,     
  dsbd8.dwBufferBytes := wsr.m_ckIn.cksize;
  dsbd8.lpwfxFormat   := wsr.m_pwfx;

  //     
  if FAILED( lpDS8.CreateSoundBuffer( dsbd8, lpSoundBuffer, nil ) ) then
     Exit;
  lpSoundBuffer._AddRef();

  //       DirectSound
  bbytes := dsbd8.dwBufferBytes;

  Result := TRUE;
end;




//------------------------------------------------------------------------------
// :      FillBuffer()
// :    DirectSound   wav-
//------------------------------------------------------------------------------
function FillBuffer( var (*   *)lpBuffer: IDirectSoundBuffer ): Boolean;
var
  pbWavData: PByte;
  cbWavSize: UINT;
  pbData:    Pointer;
  pbData2:   Pointer;
  dwLength:  DWORD;
  dwLength2: DWORD;
  nWaveFileSize: Integer;
begin
  Result := FALSE;

  pbData  := nil;
  pbData2 := nil;

  //    
  nWaveFileSize := wsr.m_ckIn.cksize;


  //    
  //    SDK
  // pbWavData = new BYTE[ nWaveFileSize ]
  //   
  GetMem( pbWavData, nWaveFileSize );
  //   : GetMem( pbWavData, Byte( nWaveFileSize ) ) -  
  //  -  !



  // ,    
  if pbWavData = nil then
     Exit;

  //      
  if not wsr.Read( nWaveFileSize, pbWavData, @cbWavSize ) then
     Exit;

  //     
  wsr.Reset();

  //   (   )
  if FAILED( lpSoundBuffer.Lock( 0, bbytes, pbData, dwLength, pbData2, dwLength2, 0 ) ) then
     Exit;

  //        DirectSound
  CopyMemory( pbData, pbWavData, bbytes );

  //    
  if FAILED( lpSoundBuffer.Unlock( pbData, bbytes, nil, 0 ) ) then
     Exit;

  pbData := nil;

  //       ,  
  FreeMem( pbWavData, nWaveFileSize );
  pbWavData := nil;                    

  Result := TRUE;
end;




//------------------------------------------------------------------------------
// :      TForm1.FormCreate()
// :   DirectSound   
//------------------------------------------------------------------------------
procedure TForm1.FormCreate(Sender: TObject);
begin
  if not InitDirectSound( Form1.Handle ) then
  begin
     MessageBox( Form1.Handle, '   DirectSound!',
                '!', MB_ICONHAND );
     ReleaseDirectSound();
     Halt;
  end;
end;




//------------------------------------------------------------------------------
// :      TForm1.btnOpenClick()
// :      wav-   UI
//------------------------------------------------------------------------------
procedure TForm1.btnOpenClick(Sender: TObject);
begin
  if odOpenFile.Execute then
  if ( CreateStaticBuffer( odOpenFile.FileName ) ) and ( FillBuffer( lpSoundBuffer ) )
  then
  begin
    edFileName.Text := odOpenFile.FileName; //    
    bbPlay.Enabled := TRUE;                 //   - 
  end;
end;




//------------------------------------------------------------------------------
// :      TForm1.bbPlayClick()
// :      
//------------------------------------------------------------------------------
procedure TForm1.bbPlayClick(Sender: TObject);
begin
   //      -    
   if chbLoop.Checked then lpSoundBuffer.Play( 0, 0, DSBPLAY_LOOPING )
                      else lpSoundBuffer.Play( 0, 0, 0 );
   btnOpen.Enabled := FALSE; //    - 
   bbPlay.Enabled  := FALSE; //   - 
   bbStop.Enabled  := TRUE;  //   - 
   Timer1.Enabled  := TRUE;  //  
end;




//------------------------------------------------------------------------------
// :      TForm1.bbStopClick()
// :      
//------------------------------------------------------------------------------
procedure TForm1.bbStopClick(Sender: TObject);
begin
  lpSoundBuffer.Stop();                  //  
  lpSoundBuffer.SetCurrentPosition( 0 ); //     
  bbStop.Enabled  := FALSE;              //  UI...
  bbPlay.Enabled  := TRUE;               // ...
  btnOpen.Enabled := TRUE;               // ...
end;




//------------------------------------------------------------------------------
// :      TForm1.sbVolumeChange()
// :     
//------------------------------------------------------------------------------
procedure TForm1.sbVolumeChange(Sender: TObject);
begin
  lpSoundBuffer.SetVolume( sbVolume.Position );
end;




//------------------------------------------------------------------------------
// :      TForm1.sbPanChange()
// :   ""      
//------------------------------------------------------------------------------
procedure TForm1.sbPanChange(Sender: TObject);
begin
  lpSoundBuffer.SetPan( sbPan.Position );
end;




//------------------------------------------------------------------------------
// :      TForm1.Timer1Timer()
// :     
//------------------------------------------------------------------------------
procedure TForm1.Timer1Timer(Sender: TObject);
var
  dwStatus: DWORD;
begin
  //   ,     dwStatus
  lpSoundBuffer.GetStatus( dwStatus );

  //    ,   UI   
  if ( dwStatus and DSBSTATUS_PLAYING ) <> DSBSTATUS_PLAYING then
  begin
    bbStop.Enabled  := FALSE; //   - 
    bbPlay.Enabled  := TRUE;  //   - 
    btnOpen.Enabled := TRUE;  //    - 
  end;
end;




//------------------------------------------------------------------------------
// :      TForm1.FormDestroy()
// :      
//------------------------------------------------------------------------------
procedure TForm1.FormDestroy(Sender: TObject);
begin
  ReleaseDirectSound();
end;

end.
